'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AggregateRelation = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _types = require('./types');

var _tcomb = require('tcomb');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var AggregateRelation = exports.AggregateRelation = function () {
  /**
   * Manages relations for aggregates
   *
   * @param {AggregateRepository} repository
   * @param {redis.client} promisified redis client
   */
  function AggregateRelation(repository, redis) {
    _classCallCheck(this, AggregateRelation);

    this.repository = repository;
    this.redis = redis;
  }

  /**
   * Finds all entities that are associated with the given relation of the given relationId
   *
   * The need to be added via addRelatedId()
   *
   * @param {String} relation
   * @param {String} relatedId
   * @returns {Promise.<Array.<AggregateRoot>>}
   */


  _createClass(AggregateRelation, [{
    key: 'findByRelatedId',
    value: function findByRelatedId(relation, relatedId) {
      (0, _tcomb.String)(relation);
      (0, _types.AggregateIdType)(relatedId);
      var self = this;
      return self.redis.smembersAsync(self.repository.aggregateAlias + ':' + relation + ':' + relatedId).map(self.repository.findById.bind(self.repository)).filter(function (model) {
        return model !== undefined;
      });
    }

    /**
     * A helper function for associating the the aggregateId with the given relatedId of the relation
     *
     * e.g. Associate the user (aggregateId='17') with the meeting (relation='meeting') of id 42 (relatedId='42')
     * Now all users of meeting 42 can be returned via findByRelatedId('meeting', '42')
     *
     * @param {String} relation
     * @param {String} relatedId
     * @param {String} aggregateId
     * @returns {Promise}
     */

  }, {
    key: 'addRelatedId',
    value: function addRelatedId(relation, relatedId, aggregateId) {
      (0, _tcomb.String)(relation);
      (0, _types.AggregateIdType)(relatedId);
      (0, _types.AggregateIdType)(aggregateId);
      var self = this;
      return self.redis.saddAsync(self.repository.aggregateAlias + ':' + relation + ':' + relatedId, aggregateId);
    }

    /**
     * A helper function for removing the the aggregateId with the given relatedId of the relation
     *
     * @param {String} relation
     * @param {String} relatedId
     * @param {String} aggregateId
     * @returns {Promise}
     */

  }, {
    key: 'removeRelatedId',
    value: function removeRelatedId(relation, relatedId, aggregateId) {
      (0, _tcomb.String)(relation);
      (0, _types.AggregateIdType)(relatedId);
      (0, _types.AggregateIdType)(aggregateId);
      var self = this;
      return self.redis.sremAsync(self.repository.aggregateAlias + ':' + relation + ':' + relatedId, aggregateId);
    }
  }]);

  return AggregateRelation;
}();