'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AggregateRepositoryType = exports.AggregateRepository = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _eventStore = require('./event-store');

var _modelEvent = require('./model-event');

var _rheactorErrors = require('@resourcefulhumans/rheactor-errors');

var _bluebird = require('bluebird');

var _aggregateRoot = require('./aggregate-root');

var _types = require('./types');

var _tcomb = require('tcomb');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @deprecated Use ImmutableAggregateRepository
 */
var AggregateRepository = exports.AggregateRepository = function () {

  /**
   * Creates a new aggregate repository
   *
   * @param {AggregateRoot} aggregateRoot
   * @param {String} aggregateAlias
   * @param {redis.client} redis
   */
  function AggregateRepository(aggregateRoot, aggregateAlias, redis) {
    _classCallCheck(this, AggregateRepository);

    this.aggregateRoot = aggregateRoot;
    this.aggregateAlias = aggregateAlias;
    this.aggregateAliasPrefix = aggregateAlias.charAt(0).toUpperCase() + aggregateAlias.slice(1);
    this.eventStore = new _eventStore.EventStore(aggregateAlias, redis);
    this.redis = redis;
  }

  /**
   * Generic method for add aggregates to the collection.
   * The repository will assign an ID to them.
   *
   * Calls applyEvent with the created event on the aggregate.
   *
   * @param {AggregateRoot} aggregate
   * @param {String} createdBy
   * @returns {Promise.<ModelEvent>}
   */


  _createClass(AggregateRepository, [{
    key: 'add',
    value: function add(aggregate, createdBy) {
      var _this = this;

      (0, _aggregateRoot.AggregateRootType)(aggregate);
      (0, _types.MaybeStringType)(createdBy);
      return this.redis.incrAsync(this.aggregateAlias + ':id').then(function (id) {
        var event = new _modelEvent.ModelEvent('' + id, _this.aggregateAliasPrefix + 'CreatedEvent', aggregate, new Date(), createdBy);
        return _this.persistEvent(event).then(function () {
          aggregate.applyEvent(event);
          return event;
        });
      });
    }

    /**
     * Generic method to persist model events
     *
     * @param {ModelEvent} modelEvent
     * @return {Promise.<ModelEvent>}
     */

  }, {
    key: 'persistEvent',
    value: function persistEvent(modelEvent) {
      (0, _modelEvent.ModelEventType)(modelEvent);
      return this.eventStore.persist(modelEvent).then(function () {
        return modelEvent;
      });
    }

    /**
     * Generic method for removing aggregates from the collection
     *
     * Calls applyEvent with the created event on the aggregate.
     *
     * @param {AggregateRoot} aggregate
     * @param {String} createdBy
     * @returns {Promise.<ModelEvent>}
     */

  }, {
    key: 'remove',
    value: function remove(aggregate, createdBy) {
      (0, _aggregateRoot.AggregateRootType)(aggregate);
      (0, _types.MaybeStringType)(createdBy);
      var event = new _modelEvent.ModelEvent(aggregate.aggregateId(), this.aggregateAliasPrefix + 'DeletedEvent', aggregate, new Date(), createdBy);
      return this.persistEvent(event).then(function () {
        aggregate.applyEvent(event);
        return event;
      });
    }

    /**
     * Generic method for loading aggregates by id
     *
     * @param {String} id
     * @returns {Promise.<AggregateRoot>} or undefined if not found
     */

  }, {
    key: 'findById',
    value: function findById(id) {
      var _this2 = this;

      (0, _types.AggregateIdType)(id);
      return this.eventStore.fetch(id).then(function (events) {
        return _this2.aggregate(events);
      }).then(function (aggregate) {
        if (!aggregate) return;
        return aggregate.isDeleted() ? undefined : aggregate;
      });
    }

    /**
     * Creates an instance of this repositories aggregate root by applying all events
     *
     * NOTE: Because the signature of the AggregateRoot's constructor may change
     * over time, we bypass the individual constructor of the Model. The model
     * is passed the the complete list of events (including the created event)
     * and must construct itthis from that list.
     * This way we can keep hard checks when constructing new model instance from
     * code "new MyExampleModel(arg)" but can handle changing payload of the created
     * event over time.
     *
     * @deprecated Because this method bypasses the Model constructor it will very likely be removed in the future.
     *
     * @param {Array.<ModelEvent>} events
     * @returns {AggregateRoot|undefined}
     */

  }, {
    key: 'aggregate',
    value: function aggregate(events) {
      (0, _modelEvent.ModelEventTypeList)(events);
      if (!events.length) {
        return;
      }
      var model = Object.create(this.aggregateRoot.prototype); // Instantiate model
      _aggregateRoot.AggregateRoot.call(model); // Bypass the model constructor, but init necessary data
      events.map(model.applyEvent.bind(model));
      return model;
    }

    /**
     * Returns all entities
     *
     * NOTE: naively returns all entities by fetching them one by one by ID starting
     * from 1 to the current max id.
     *
     * @returns {Promise.<Array.<AggregateRoot>>}
     */

  }, {
    key: 'findAll',
    value: function findAll() {
      var _this3 = this;

      return this.redis.getAsync(this.aggregateAlias + ':id').then(function (maxId) {
        var promises = [];
        for (var i = 1; i <= maxId; i++) {
          promises.push(_this3.findById('' + i));
        }
        return _bluebird.Promise.all(promises);
      }).filter(function (item) {
        return item !== undefined;
      });
    }

    /**
     * Generic method for loading aggregates by id
     *
     * @param {String} id
     * @returns {Promise.<AggregateRoot>}
     * @throws {EntryNotFoundError} if entity is not found
     */

  }, {
    key: 'getById',
    value: function getById(id) {
      var _this4 = this;

      (0, _types.AggregateIdType)(id);
      return this.eventStore.fetch(id).then(function (events) {
        return _this4.eventsToAggregate(id, events);
      });
    }
  }, {
    key: 'eventsToAggregate',
    value: function eventsToAggregate(id, events) {
      (0, _types.AggregateIdType)(id);
      (0, _modelEvent.ModelEventTypeList)(events);
      var aggregate = this.aggregate(events);
      if (!aggregate) {
        throw new _rheactorErrors.EntryNotFoundError(this.aggregateAlias + ' with id "' + id + '" not found.');
      }
      if (aggregate.isDeleted()) {
        throw new _rheactorErrors.EntryDeletedError(this.aggregateAlias + ' with id "' + id + '" is deleted.', aggregate);
      }
      return aggregate;
    }

    /**
     * Returns true if x is of type AggregateRepository
     *
     * @param {object} x
     * @returns {boolean}
     */

  }], [{
    key: 'is',
    value: function is(x) {
      return x instanceof AggregateRepository || x && x.constructor && x.constructor.name === AggregateRepository.name && 'aggregateRoot' in x && 'aggregateAlias' in x && 'aggregateAliasPrefix' in x && 'eventStore' in x && 'redis' in x;
    }
  }]);

  return AggregateRepository;
}();

var AggregateRepositoryType = exports.AggregateRepositoryType = (0, _tcomb.irreducible)('AggregateRepositoryType', AggregateRepository.is);