'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.AggregateRootType = exports.AggregateRoot = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _types = require('./types');

var _tcomb = require('tcomb');

var _modelEvent = require('./model-event');

var _rheactorErrors = require('@resourcefulhumans/rheactor-errors');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @deprecated Use ImmutableAggregateRoot
 */
var AggregateRoot = exports.AggregateRoot = function () {
  /**
   * Base class for aggregates
   */
  function AggregateRoot() {
    _classCallCheck(this, AggregateRoot);

    Object.defineProperty(this, '$aggregateMeta', {
      value: {
        id: null,
        version: null,
        deleted: false,
        createdAt: null,
        updatedAt: null,
        deletedAt: null
      }
    });
  }

  /**
   * @param {String} aggregateId
   * @param {Date} createdAt
   */


  _createClass(AggregateRoot, [{
    key: 'persisted',
    value: function persisted(aggregateId) {
      var createdAt = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : new Date();

      (0, _types.AggregateIdType)(aggregateId);
      (0, _tcomb.Date)(createdAt);
      this.$aggregateMeta.id = aggregateId;
      this.$aggregateMeta.version = 1;
      this.$aggregateMeta.createdAt = createdAt;
    }

    /**
     * @param {Date} updatedAt
     * @returns {Number}
     */

  }, {
    key: 'updated',
    value: function updated() {
      var updatedAt = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

      (0, _tcomb.Date)(updatedAt);
      this.$aggregateMeta.updatedAt = updatedAt;
      return ++this.$aggregateMeta.version;
    }

    /**
     * @param {Date} deletedAt
     * @returns {number}
     */

  }, {
    key: 'deleted',
    value: function deleted() {
      var deletedAt = arguments.length > 0 && arguments[0] !== undefined ? arguments[0] : new Date();

      (0, _tcomb.Date)(deletedAt);
      this.$aggregateMeta.deletedAt = deletedAt;
      this.$aggregateMeta.deleted = true;
      return ++this.$aggregateMeta.version;
    }

    /**
     * Returns the aggregate version
     *
     * @returns {number} Version of the aggregation
     */

  }, {
    key: 'aggregateVersion',
    value: function aggregateVersion() {
      return this.$aggregateMeta.version;
    }

    /**
     * Returns the aggregate id
     *
     * @returns {String} ID of the aggregation
     */

  }, {
    key: 'aggregateId',
    value: function aggregateId() {
      return this.$aggregateMeta.id;
    }

    /**
     * Returns if the aggregate is deleted
     *
     * @returns {Boolean}
     */

  }, {
    key: 'isDeleted',
    value: function isDeleted() {
      return this.$aggregateMeta.deleted;
    }

    /**
     * Returns the timestamp when the aggregate was created
     *
     * @returns {Number}
     */

  }, {
    key: 'createdAt',
    value: function createdAt() {
      return this.$aggregateMeta.createdAt;
    }

    /**
     * Returns the timestamp when the aggregate was updated
     *
     * @returns {Number|null}
     */

  }, {
    key: 'updatedAt',
    value: function updatedAt() {
      return this.$aggregateMeta.updatedAt;
    }

    /**
     * Returns the timestamp when the aggregate was modified the last time, which is the latest value of
     * createdAt, updatedAt or deletedAt
     *
     * @returns {Number}
     */

  }, {
    key: 'modifiedAt',
    value: function modifiedAt() {
      if (this.$aggregateMeta.deletedAt) {
        return this.$aggregateMeta.deletedAt;
      }
      if (this.$aggregateMeta.updatedAt) {
        return this.$aggregateMeta.updatedAt;
      }
      return this.$aggregateMeta.createdAt;
    }

    /**
     * Returns the timestamp when the aggregate was deleted
     *
     * @returns {Number|null}
     */

  }, {
    key: 'deletedAt',
    value: function deletedAt() {
      return this.$aggregateMeta.deletedAt;
    }

    /**
     * Applies the event to the aggregate.
     * Should return the modified aggregate.
     *
     * @param {ModelEvent} event
     * @returns {AggregateRoot}
     */

  }, {
    key: 'applyEvent',
    value: function applyEvent(event) {
      (0, _modelEvent.ModelEventType)(event);
      throw new _rheactorErrors.UnhandledDomainEventError(event.name + ' on ' + this.constructor.name);
    }

    /**
     * Returns true if x is of type AggregateRoot
     *
     * @param {object} x
     * @returns {boolean}
     */

  }], [{
    key: 'is',
    value: function is(x) {
      return x instanceof AggregateRoot || x && x.constructor && x.constructor.name === AggregateRoot.name && 'persisted' in x && 'updated' in x && 'deleted' in x && 'aggregateVersion' in x && 'aggregateId' in x && 'isDeleted' in x && 'createdAt' in x && 'modifiedAt' in x && 'updatedAt' in x && 'deletedAt' in x;
    }
  }]);

  return AggregateRoot;
}();

var AggregateRootType = exports.AggregateRootType = (0, _tcomb.irreducible)('AggregateRootType', AggregateRoot.is);