'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ImmutableAggregateRepositoryType = exports.ImmutableAggregateRepository = undefined;

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

var _eventStore = require('./event-store');

var _modelEvent = require('./model-event');

var _rheactorErrors = require('@resourcefulhumans/rheactor-errors');

var _bluebird = require('bluebird');

var _immutableAggregateRoot = require('./immutable-aggregate-root');

var _types = require('./types');

var _tcomb = require('tcomb');

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var ImmutableAggregateRepository = exports.ImmutableAggregateRepository = function () {

  /**
   * Creates a new aggregate repository
   *
   * @param {ImmutableAggregateRoot} {applyEvent: {Function}}
   * @param {String} alias
   * @param {redis.client} redis
   */
  function ImmutableAggregateRepository(_ref, alias, redis) {
    var applyEvent = _ref.applyEvent;

    _classCallCheck(this, ImmutableAggregateRepository);

    (0, _tcomb.Function)(applyEvent, ['ImmutableAggregateRepository', 'root:ImmutableAggregateRoot']);
    (0, _tcomb.String)(alias, ['ImmutableAggregateRepository', 'alias:String']);
    this.applyEvent = applyEvent;
    this.alias = alias;
    this.prefix = alias.charAt(0).toUpperCase() + alias.slice(1);
    this.eventStore = new _eventStore.EventStore(alias, redis);
    this.redis = redis;
  }

  /**
   * Generic method for add aggregates to the collection.
   * The repository will assign an ID to them.
   *
   * Calls applyEvent with the created event on the aggregate.
   *
   * @param {Object} data
   * @param {String} createdBy
   * @returns {Promise.<ModelEvent>}
   */


  _createClass(ImmutableAggregateRepository, [{
    key: 'add',
    value: function add(data, createdBy) {
      var _this = this;

      (0, _tcomb.Object)(data, ['ImmutableAggregateRepository', 'add()', 'data:Object']);
      (0, _types.MaybeStringType)(createdBy, ['ImmutableAggregateRepository', 'add()', 'createdBy:?String']);
      return this.redis.incrAsync(this.alias + ':id').then(function (id) {
        return _this.persistEvent(new _modelEvent.ModelEvent('' + id, _this.prefix + 'CreatedEvent', data, new Date(), createdBy));
      });
    }

    /**
     * Generic method to persist model events
     *
     * @param {ModelEvent} modelEvent
     * @return {Promise.<ModelEvent>}
     */

  }, {
    key: 'persistEvent',
    value: function persistEvent(modelEvent) {
      (0, _modelEvent.ModelEventType)(modelEvent, ['ImmutableAggregateRepository', 'persistEvent()', 'modelEvent:ModelEvent']);
      return this.eventStore.persist(modelEvent).then(function () {
        return modelEvent;
      });
    }

    /**
     * Generic method for removing aggregates from the collection
     *
     * @param {AggregateRoot} aggregate
     * @param {String} createdBy
     * @returns {Promise.<ModelEvent>}
     */

  }, {
    key: 'remove',
    value: function remove(aggregate, createdBy) {
      (0, _immutableAggregateRoot.ImmutableAggregateRootType)(aggregate, ['ImmutableAggregateRepository', 'remove()', 'aggregate:ImmutableAggregateRoot']);
      (0, _types.MaybeStringType)(createdBy, ['ImmutableAggregateRepository', 'remove()', 'createdBy:?String']);
      return this.persistEvent(new _modelEvent.ModelEvent(aggregate.meta.id, this.prefix + 'DeletedEvent', aggregate, new Date(), createdBy));
    }

    /**
     * Generic method for loading aggregates by id
     *
     * @param {String} id
     * @returns {Promise.<AggregateRoot>} or undefined if not found
     */

  }, {
    key: 'findById',
    value: function findById(id) {
      var _this2 = this;

      (0, _types.AggregateIdType)(id, ['ImmutableAggregateRepository', 'findById()', 'id:AggregateId']);
      return this.eventStore.fetch(id).reduce(function (aggregate, event) {
        return _this2.applyEvent(event, aggregate === false ? undefined : aggregate);
      }, false).then(function (aggregate) {
        if (!aggregate) return;
        return aggregate.meta.isDeleted ? undefined : aggregate;
      });
    }

    /**
     * Returns all entities
     *
     * NOTE: naively returns all entities by fetching them one by one by ID starting
     * from 1 to the current max id.
     *
     * @returns {Promise.<Array.<AggregateRoot>>}
     */

  }, {
    key: 'findAll',
    value: function findAll() {
      var _this3 = this;

      return this.redis.getAsync(this.alias + ':id').then(function (maxId) {
        var promises = [];
        for (var i = 1; i <= maxId; i++) {
          promises.push(_this3.findById('' + i));
        }
        return _bluebird.Promise.all(promises);
      }).filter(function (item) {
        return item !== undefined;
      });
    }

    /**
     * Generic method for loading aggregates by id
     *
     * @param {String} id
     * @returns {Promise.<AggregateRoot>}
     * @throws {EntryNotFoundError} if entity is not found
     */

  }, {
    key: 'getById',
    value: function getById(id) {
      var _this4 = this;

      (0, _types.AggregateIdType)(id, ['ImmutableAggregateRepository', 'getById()', 'id:AggregateId']);
      return this.eventStore.fetch(id).reduce(function (aggregate, event) {
        return _this4.applyEvent(event, aggregate === false ? undefined : aggregate);
      }, false).then(function (aggregate) {
        if (!aggregate) {
          throw new _rheactorErrors.EntryNotFoundError(_this4.alias + ' with id "' + id + '" not found.');
        }
        if (aggregate.meta.isDeleted) {
          throw new _rheactorErrors.EntryDeletedError(_this4.alias + ' with id "' + id + '" is deleted.', aggregate);
        }
        return aggregate;
      });
    }

    /**
     * Returns true if x is of type ImmutableAggregateRepository
     *
     * @param {object} x
     * @returns {boolean}
     */

  }], [{
    key: 'is',
    value: function is(x) {
      return x instanceof ImmutableAggregateRepository || x && x.constructor && x.constructor.name === ImmutableAggregateRepository.name && 'root' in x && 'alias' in x && 'prefix' in x && 'eventStore' in x && 'redis' in x;
    }
  }]);

  return ImmutableAggregateRepository;
}();

var ImmutableAggregateRepositoryType = exports.ImmutableAggregateRepositoryType = (0, _tcomb.irreducible)('ImmutableAggregateRepositoryType', ImmutableAggregateRepository.is);